@@ -63,16 +63,21 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
                 if self.current_env_vars[v] is not None:
                     os.environ[v] = self.current_env_vars[v]
 
-    def test_xpath_tokenizer2(self):
+    def test_xpath_tokenizer(self):
+        super(XPath2ParserTest, self).test_xpath_tokenizer()
         self.check_tokenizer("(: this is a comment :)",
                              ['(:', '', 'this', '', 'is', '', 'a', '', 'comment', '', ':)'])
         self.check_tokenizer("last (:", ['last', '', '(:'])
 
-    def test_token_tree2(self):
+    def test_token_tree(self):
+        super(XPath2ParserTest, self).test_token_tree()
         self.check_tree('(1 + 6, 2, 10 - 4)', '(, (, (+ (1) (6)) (2)) (- (10) (4)))')
         self.check_tree('/A/B2 union /A/B1', '(union (/ (/ (A)) (B2)) (/ (/ (A)) (B1)))')
+        self.check_tree("//text/(preceding-sibling::text)[1]",
+                        '(/ (// (text)) ([ (preceding-sibling (text)) (1)))')
 
-    def test_token_source2(self):
+    def test_token_source(self):
+        super(XPath2ParserTest, self).test_token_source()
         self.check_source("(5, 6) instance of xs:integer+", '(5, 6) instance of xs:integer+')
         self.check_source("$myaddress treat as element(*, USAddress)",
                           "$myaddress treat as element(*, USAddress)")
@@ -247,6 +252,41 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         self.check_value("4 gt ()")
         self.check_value("() eq ()")  # Equality of empty sequences is also an empty sequence
 
+    def test_comparison_in_expression(self):
+        context = XPathContext(self.etree.XML('<value>false</value>'))
+        self.check_value("(. = 'false') = (. = 'false')", True, context)
+        self.check_value("(. = 'asdf') != (. = 'false')", True, context)
+
+    def test_boolean_evaluation_in_selector(self):
+        context = XPathContext(self.etree.XML("""
+<collection>
+    <book>
+        <available>true</available>
+        <price>10.0</price>
+    </book>
+    <book>
+        <available>1</available>
+        <price>10.0</price>
+    </book>
+    <book>
+        <available>false</available>
+        <price>5.0</price>
+    </book>
+    <book>
+        <available>0</available>
+        <price>5.0</price>
+    </book>
+</collection>
+        """))
+        expr = "sum(//price)"
+        self.check_value("sum(//price)", 30, context)
+        self.check_value("sum(//price[../available = 'true'])", 10, context)
+        self.check_value("sum(//price[../available = 'false'])", 5, context)
+        self.check_value("sum(//price[../available = '1'])", 10, context)
+        self.check_value("sum(//price[../available = '0'])", 5, context)
+        self.check_value("sum(//price[../available = true()])", 20, context)
+        self.check_value("sum(//price[../available = false()])", 10, context)
+
     def test_comparison_of_sequences(self):
         super(XPath2ParserTest, self).test_comparison_of_sequences()
 
@@ -299,36 +339,34 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
                     <xs:attribute name="max" type="xs:int"/>
                   </xs:complexType>
                 </xs:schema>""")
-            self.parser.schema = xmlschema.xpath.XMLSchemaProxy(schema, schema.elements['root'])
 
-            root = self.etree.XML('<root>11</root>')
-            self.check_value('. le 10', False, context=XPathContext(root))
-            self.check_value('. le 20', True, context=XPathContext(root))
+            with self.schema_bound_parser(schema.elements['root'].xpath_proxy):
+                root = self.etree.XML('<root>11</root>')
+                self.check_value('. le 10', False, context=XPathContext(root))
+                self.check_value('. le 20', True, context=XPathContext(root))
 
-            root = self.etree.XML('<root>eleven</root>')
-            with self.assertRaises(TypeError) as err:
-                self.check_value('. le 10', context=XPathContext(root))
-            self.assertIn('XPTY0004', str(err.exception))  # Dynamic context error
+                root = self.etree.XML('<root>eleven</root>')
+                with self.assertRaises(TypeError) as err:
+                    self.check_value('. le 10', context=XPathContext(root))
+                self.assertIn('XPTY0004', str(err.exception))  # Dynamic context error
 
-            root = self.etree.XML('<value>12</value>')
-            with self.assertRaises(TypeError) as err:
-                self.check_value('. le "11"', context=XPathContext(root))
-            self.assertIn('XPTY0004', str(err.exception))  # Static schema context error
+                root = self.etree.XML('<value>12</value>')
+                with self.assertRaises(TypeError) as err:
+                    self.check_value('. le "11"', context=XPathContext(root))
+                self.assertIn('XPTY0004', str(err.exception))  # Static schema context error
 
-            with self.assertRaises(TypeError) as err:
-                self.check_value('. le 10', context=XPathContext(root))
-            self.assertIn('FORG0006', str(err.exception))  # Dynamic context error
+                with self.assertRaises(TypeError) as err:
+                    self.check_value('. le 10', context=XPathContext(root))
+                self.assertIn('FORG0006', str(err.exception))  # Dynamic context error
 
             schema = xmlschema.XMLSchema("""
                 <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
                   <xs:element name="root" type="xs:anyType"/>
                 </xs:schema>""")
-            self.parser.schema = xmlschema.xpath.XMLSchemaProxy(schema, schema.elements['root'])
 
-            root = self.etree.XML('<root>15</root>')
-            self.check_value('. le "11"', False, context=XPathContext(root))
-
-            self.parser.schema = None
+            with self.schema_bound_parser(schema.elements['root'].xpath_proxy):
+                root = self.etree.XML('<root>15</root>')
+                self.check_value('. le "11"', False, context=XPathContext(root))
 
         root = self.etree.XML('<root><a>1</a><a>10</a><a>30</a><a>50</a></root>')
         self.check_selector("a = (1 to 30)", root, True)
@@ -510,12 +548,11 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
                   </xs:complexType>
                 </xs:schema>""")
 
-            self.parser.schema = xmlschema.xpath.XMLSchemaProxy(schema, schema.elements['A'])
-            self.check_select("attribute(a, xs:int)", ['10'], context)
-            self.check_select("attribute(*, xs:int)", {'10', '20'}, context)
-            self.check_select("attribute(a, xs:string)", [], context)
-            self.check_select("attribute(*, xs:string)", [], context)
-            self.parser.schema = None
+            with self.schema_bound_parser(schema.elements['A'].xpath_proxy):
+                self.check_select("attribute(a, xs:int)", ['10'], context)
+                self.check_select("attribute(*, xs:int)", {'10', '20'}, context)
+                self.check_select("attribute(a, xs:string)", [], context)
+                self.check_select("attribute(*, xs:string)", [], context)
 
     def test_node_and_node_accessors(self):
         element = self.etree.Element('schema')
@@ -627,6 +664,112 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         self.check_value('fn:concat($word, fn:lower-case(10))', TypeError)
         self.check_value('fn:concat($unknown, fn:lower-case(10))', TypeError)
 
+    def test_instance_of_expression(self):
+        element = self.etree.Element('schema')
+
+        # Test cases from https://www.w3.org/TR/xpath20/#id-instance-of
+        self.check_value("5 instance of xs:integer", True)
+        self.check_value("5 instance of xs:decimal", True)
+        self.check_value("9.0 instance of xs:integer", False)
+        self.check_value("(5, 6) instance of xs:integer+", True)
+
+        context = XPathContext(element)
+        self.check_value(". instance of element()", True, context)
+        context.item = None
+        self.check_value(". instance of element()", False, context)
+
+        self.check_value("(5, 6) instance of xs:integer", False)
+        self.check_value("(5, 6) instance of xs:integer*", True)
+        self.check_value("(5, 6) instance of xs:integer?", False)
+
+        self.check_value("5 instance of empty-sequence()", False)
+        self.check_value("() instance of empty-sequence()", True)
+
+    def test_treat_as_expression(self):
+        element = self.etree.Element('schema')
+        context = XPathContext(element)
+
+        self.check_value("5 treat as xs:integer", [5])
+        self.check_value("5 treat as xs:string", ElementPathTypeError)
+        self.check_value("5 treat as xs:decimal", [5])
+        self.check_value("(5, 6) treat as xs:integer+", [5, 6])
+        self.check_value(". treat as element()", [element], context)
+
+        self.check_value("(5, 6) treat as xs:integer", ElementPathTypeError)
+        self.check_value("(5, 6) treat as xs:integer*", [5, 6])
+        self.check_value("(5, 6) treat as xs:integer?", ElementPathTypeError)
+
+        self.check_value("5 treat as empty-sequence()", ElementPathTypeError)
+        self.check_value("() treat as empty-sequence()", [])
+
+    def test_castable_expression(self):
+        self.check_value("5 castable as xs:integer", True)
+        self.check_value("'5' castable as xs:integer", True)
+        self.check_value("'hello' castable as xs:integer", False)
+        self.check_value("('5', '6') castable as xs:integer", False)
+        self.check_value("() castable as xs:integer", False)
+        self.check_value("() castable as xs:integer?", True)
+
+        self.check_value("'NaN' castable as xs:double", True)
+        self.check_value("'None' castable as xs:double", False)
+        self.check_value("'NaN' castable as xs:float", True)
+        self.check_value("'NaN' castable as xs:integer", False)
+
+    def test_cast_expression(self):
+        self.check_value("5 cast as xs:integer", 5)
+        self.check_value("'5' cast as xs:integer", 5)
+        self.check_value("'hello' cast as xs:integer", ValueError)
+        self.check_value("('5', '6') cast as xs:integer", TypeError)
+        self.check_value("() cast as xs:integer", TypeError)
+        self.check_value("() cast as xs:integer?", [])
+        self.check_value('"1" cast as xs:boolean', True)
+        self.check_value('"0" cast as xs:boolean', False)
+
+    def test_logical_expressions_(self):
+        super(XPath2ParserTest, self).test_logical_expressions()
+
+        if xmlschema is not None:
+            schema = xmlschema.XMLSchema("""
+                <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
+                  <xs:element name="root">
+                    <xs:complexType>
+                      <xs:sequence/>
+                      <xs:attribute name="a" type="xs:integer"/>      
+                      <xs:attribute name="b" type="xs:integer"/>
+                    </xs:complexType>
+                  </xs:element>
+                </xs:schema>""")
+
+            with self.schema_bound_parser(schema.elements['root'].xpath_proxy):
+                root_token = self.parser.parse("(@a and not(@b)) or (not(@a) and @b)")
+                context = XPathContext(self.etree.XML('<root a="10" b="0"/>'))
+                self.assertTrue(root_token.evaluate(context=context) is True)
+                context = XPathContext(self.etree.XML('<root a="10" b="1"/>'))
+                self.assertTrue(root_token.evaluate(context=context) is False)
+                context = XPathContext(self.etree.XML('<root a="10"/>'))
+                self.assertTrue(root_token.evaluate(context=context) is True)
+                context = XPathContext(self.etree.XML('<root a="0" b="10"/>'))
+                self.assertTrue(root_token.evaluate(context=context) is True)
+                context = XPathContext(self.etree.XML('<root b="0"/>'))
+                self.assertTrue(root_token.evaluate(context=context) is False)
+
+    def test_element_decimal_cast(self):
+        root = self.etree.XML('''
+        <books>
+            <book><isbn>1558604820</isbn><price>12.50</price></book>
+            <book><isbn>1558604820</isbn><price>13.50</price></book>
+            <book><isbn>1558604820</isbn><price>-0.1</price></book>
+        </books>''')
+        expected_values = [Decimal('12.5'), Decimal('13.5'), Decimal('-0.1')]
+        self.assertEqual(3, len(select(root, "//book")))
+        for book in iter_select(root, "//book"):
+            context = XPathContext(root=root, item=book)
+            root_token = self.parser.parse("xs:decimal(price)")
+            self.assertEqual(expected_values.pop(0), root_token.evaluate(context))
+
+    def test_element_decimal_comparison_after_round(self):
+        self.check_value('xs:decimal(0.36) = round(0.36*100) div 100', True)
+
 
 @unittest.skipIf(lxml_etree is None, "The lxml library is not installed")
 class LxmlXPath2ParserTest(XPath2ParserTest):
